
#include<linux/init.h>
#include<linux/module.h>
#include <linux/kernel.h>
#include <linux/poll.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/wait.h>
#include <linux/interrupt.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/cacheflush.h>
#include <asm/ioctl.h>
#include "gpio_vsc.h"
#include "gpio_info.h"

#define GPIO_REG_CHR_DEV_FAILS 		0xAA
#define GPIO_MEM_CHECK_REG_FAILS	0xBB
#define GPIO_REQ_MEM_REG_FAILS		0xCC
#define GPIO_MEM_REMAP_FAILS		0x0A
#define SET_GPIO_OPTION			3

#define IRQ_GPIO			6
#define INT_OBSERVE_REG			((volatile unsigned long*) 0xB02008F8)	/* Bit 20 of this register holds the Interrupt status of gpios */
#define GPIO_OBSERVE_INT		1<<20
DECLARE_WAIT_QUEUE_HEAD(gpio_intr_wq);

static int interrupt_pin = 17;
static int interrupt_port = 6;
uint32_t GPIO_interrupt_status[MAX_GPIO_PORTS] = {0xB020080C, 0xB020081C, 0xB020082C, 0xB020083C, 0xB020084C};

uint32_t GPIOPortBaseOffset[MAX_GPIO_PORTS] = {0x0000,
												0x0040,
												0x0080,
												0x00C0,
												0x0100
											  };

extern GPIO_PORT_PIN_MAPPING GpioPortPinMap[MAX_GPIOS];
extern GPIO_MAP_TABLE Gpio_Table[MAX_GPIOS];
extern int write_init_st(int option,int value);
/*****Prototye Functions*************************/
static int    gpio_vsc_init (void );
static void gpio_vsc_exit (void);
static  int  gpio_vsc_open (struct inode *, struct file *);
static  int  gpio_vsc_release (struct inode *, struct file *);
static  int  gpio_vsc_read (struct file *, char *, size_t, loff_t *);
static  int  gpio_vsc_write (struct file *, const char *, size_t, loff_t *);
static  int  gpio_vsc_ioctl (struct inode *, struct file *, uint, ulong);

static irqreturn_t gpio_intr_handler(int this_irq, void *dev_id);
static irqreturn_t gpio_process_handler (void);

//int gpio_vsc_unreg_intr (void * gpio_data, void *gpin_data);

/*************GPIO Private Data ****************/
/* Gpio Driver Private data */
typedef struct {

	int (*readgpio)  ( void* gpio_data , void* gpin_data );
	int (*writegpio) ( void* gpio_data , void* gpin_data );
	int (*getdir)    ( void* gpio_data , void* gpin_data );
	int (*setdir)    ( void* gpio_data , void* gpin_data );
	int (*set_intr_state)    ( void* gpio_data , void* gpin_data );
	int (*get_intr_state)    ( void* gpio_data , void* gpin_data );
	int (*wait_for_intr)		( void* gpio_data, void* gpin_data);
//	int (*unregintr)		( void* gpio_data, void* gpin_data);	
			
    //int (*cleanup)   ( void *gpiodata); 
    //void* gpio_data ;
}  __attribute__((packed)) gpio_drv_t; 


static struct file_operations gpio_vsc_ops;
static gpio_dev_t  gpio_dev;
static short init_gpio_status=0;
int gpio_id =3;


static void init_gpio_table(gpio_dev_t *dev)
{
	int i = 0;
	Gpio_data_t pin_data;
 
	//printk("init_gpio_table\n");
	/* Set Default Values */
//	for (i=0; i<MAX_GPIOS;i++)
	/* Initialize only first 5 (0-4) ports right now.
	The reason is that some of the SGPIO lines are directly
	being manipulated by other modules  */
	
	for (i=0; i<MAX_GPIOS;i++)
	{
		if (i == GPIO_39) //Console Switch. Don't touch
		{
			continue;
		}
		pin_data.PinNum = i;
		pin_data.data = Gpio_Table[i].Direction;

		/* Set the direction first  */
		vsc_setdir(dev, &pin_data);

	        /* If the direction is Output initilalize with the value in the table  */
		if (Gpio_Table[i].Direction == GPIO_DIR_OUT)
		{
			pin_data.data = Gpio_Table[i].Value;
			vsc_writegpio(dev,&pin_data);
		}
	}
}

static irqreturn_t gpio_intr_handler(int this_irq, void *dev_id)
{
		uint32_t ior;
		irqreturn_t vsc_irq = IRQ_NONE;

		ior = (gpio_read_reg((uint32_t)INT_OBSERVE_REG));
		
		/* Check if the interrupt is generated by GPIO */
		if(ior & GPIO_OBSERVE_INT) {
//			printk(KERN_ALERT "Received IRQ \n");
			if(IRQ_HANDLED == gpio_process_handler()) {
				vsc_irq = IRQ_HANDLED;
//				printk(KERN_ALERT "%d %d \n", interrupt_port, interrupt_pin);	
				wake_up_interruptible(&gpio_intr_wq);		
			}
//		printk(KERN_ALERT "%d %d \n", interrupt_port, interrupt_pin);	
		}

		return vsc_irq;
		
}

#if 1
irqreturn_t gpio_process_handler(void)
{
	int i = 0;
	int j = 0;
	unsigned long gpio_compare_reg = 0;
	unsigned long gpio_int_status_reg = 0;
//	for(i=(MAX_GPIO_PORTS-1); i > -1; i--) /* For reverse port search */
	for(i=0; i < MAX_GPIO_PORTS; i++)
	{
//		printk(KERN_ALERT "gpio_int_status_reg = %4lx\naddress = %4lx\n", gpio_read_reg(GPIO_interrupt_status[i]), (unsigned long)GPIO_interrupt_status[i]);
		if(0 != (gpio_int_status_reg = gpio_read_reg(GPIO_interrupt_status[i])))
		{
			interrupt_port = i;
//			i = -1; 		/* For reverse port search */
			i = MAX_GPIO_PORTS;
			for(j=0;j < 16; j++)
			{
				gpio_compare_reg = (gpio_int_status_reg & (0x0001 << j));
				if(0 != gpio_compare_reg)
				{
					interrupt_pin = j;
					j = 16;
				}
			}
		}
	}
	
//	printk(KERN_ALERT "gpio_read_reg(GPIO_interrupt_status[%d])= %4lx\naddress = %4lx\n",interrupt_port, gpio_read_reg(GPIO_interrupt_status[interrupt_port]), (unsigned long)GPIO_interrupt_status[interrupt_port]);
//	printk(KERN_ALERT "gpio_int_status_reg = %4lx\n", gpio_int_status_reg);
	
			
//	printk(KERN_ALERT "interrupt_port = %d interrup_pin = %d\n",interrupt_port, interrupt_pin);
//	printk(KERN_ALERT "interrupt_pin = %d\n",interrupt_pin);

	gpio_write_reg(GPIO_interrupt_status[interrupt_port],gpio_compare_reg);
//	printk(KERN_ALERT "reg clear - gpio_int_status_reg = %4lx\n", gpio_read_reg(GPIO_interrupt_status[interrupt_port]));
//	printk(KERN_ALERT "IRQ_HANDLED \n");
//	wake_up_interruptible(&gpio_intr_wq);	

	return IRQ_HANDLED;
}
#endif


int __init  
gpio_vsc_init(void)
{
	int  err, ret=0;
	uint32_t gpio_base;  
	gpio_dev_t  * dev= &gpio_dev; 
	int retval = 0;

        	
	printk(KERN_ALERT " VSC Gpio Init \n ");

	memset(&gpio_vsc_ops,0,sizeof(gpio_vsc_ops));
	gpio_vsc_ops.ioctl   = gpio_vsc_ioctl;
	gpio_vsc_ops.open    = gpio_vsc_open;
	gpio_vsc_ops.release = gpio_vsc_release;
	gpio_vsc_ops.read    = gpio_vsc_read;
	gpio_vsc_ops.write   = gpio_vsc_write;
	gpio_vsc_ops.owner   = THIS_MODULE;

	/*********Register Character Device ********************/

	if ((err = register_chrdev(GPIO_MAJOR_NO, NAME, &gpio_vsc_ops)) < 0)
	{
		printk (KERN_ERR "failed to register device (err %d)\n", err);
		init_gpio_status=GPIO_REG_CHR_DEV_FAILS;
		ret = -ENODEV;
		goto fail;
	} 

	/* Remap registers */
	gpio_base = GPIO_BASE_OFFSET;
	if ((dev->gpio_addr = ioremap_nocache(gpio_base, GPIO_SIZE)) == NULL) 
	{
		printk(KERN_ERR "failed to map GPIO  IO space %08x-%08x to memory",gpio_base, gpio_base + GPIO_SIZE - 1);
		init_gpio_status=GPIO_MEM_REMAP_FAILS;
		ret = -EIO;
		goto fail;
	}

	/* check gpio memory region */
	if ((err = check_mem_region(gpio_base, GPIO_SIZE)) < 0)  
	{
		printk(KERN_ERR "GPIO IO space %08x-%08x already in use (err %d)",gpio_base, gpio_base + GPIO_SIZE - 1, err);
		init_gpio_status=GPIO_MEM_CHECK_REG_FAILS;
		ret = -EBUSY;
		goto fail;
	}

	/* request for the memory region */	
	if (request_mem_region(gpio_base, GPIO_SIZE, NAME)  == NULL) 
	{ 
		printk(KERN_ERR "IO space %08x-%08x gpio request mem region failed (err %d)",gpio_base, gpio_base + GPIO_SIZE - 1, err);
		init_gpio_status=GPIO_REQ_MEM_REG_FAILS;
		ret = -ENODEV;
		goto fail;

	}


	/* Initialize GPIO lines according to the GPIO table */
	init_gpio_table(dev);
	write_init_st(SET_GPIO_OPTION,init_gpio_status);

	if ((retval= request_irq(IRQ_GPIO, gpio_intr_handler, IRQF_SHARED, "gpio-vsc452", &gpio_id))< 0 )
	{
		printk(KERN_ERR "Retval : %d\n",retval);
		printk(KERN_ERR "Reqt for irq %d failed\n", IRQ_GPIO);
		ret= -ENODEV;
		goto fail;
	}

	printk(KERN_ALERT "GPIO Module was  sucessully loaded \n");
	return ret;	
fail:
	gpio_vsc_exit();
	return ret;
  
}

	
static void  
gpio_vsc_exit(void)
{
	gpio_dev_t  * dev= &gpio_dev; 
	uint32_t gpio_base;

	printk(KERN_ALERT " VSC452 Gpio Exit \n ");
	gpio_base = GPIO_BASE_OFFSET;

	if(init_gpio_status==0)
	{	
		init_gpio_status=0xff;
	}
	write_init_st(SET_GPIO_OPTION,init_gpio_status);
	
	/* 
	* Re-Initialize GPIO lines according to the GPIO table .
	* This is done to avoid the kernel panic caused when some GPIO pins  made as interrupt driven pin and waiting in 
	* wait_event_interruptible and these gpio are not cleared/restored back to default state when the SP reboots. 
 	*/
	init_gpio_table(dev);

	/* release the memory occupied */
	release_mem_region(gpio_base, GPIO_SIZE); 

	/* unregister the device */
	unregister_chrdev(GPIO_MAJOR_NO, NAME);

	/* Character Device unmapping   */ 
	if (dev->gpio_addr) 
	{
		iounmap(dev->gpio_addr); 
		dev->gpio_addr = NULL;
	}

	/* Free IRQ */
	free_irq(IRQ_GPIO, &gpio_id);

	printk (KERN_ALERT "GPIO Driver is unloaded sucessfully.\n");
	return;
	
 
}

/*iowrite ****/
inline void gpio_write_reg(unsigned long addr, unsigned long val)
{
        *((volatile uint32_t*)addr) = val;
}	

/*********ioread*************/
inline unsigned long gpio_read_reg(unsigned long addr )
{
        return ((*(volatile uint32_t*)addr));
}


static  int
gpio_vsc_open (struct inode *inode, struct file *file)
{
	gpio_drv_t *drv;
		
	//Alloacte the memeory for GPIO Private data
	drv = (gpio_drv_t *) kmalloc (sizeof(gpio_drv_t),GFP_KERNEL); 
	memset(drv,0,sizeof(gpio_drv_t));

	drv->readgpio=vsc_readgpio;
	drv->writegpio=vsc_writegpio;
	drv->getdir=vsc_getdir;
	drv->setdir=vsc_setdir;
	drv->set_intr_state=set_input_intr_state;
//	drv->get_intr_state=get_input_intr_state;
	drv->wait_for_intr=wait_input_intr;
//	drv->unregintr;
	//Storing GPIO Private data 
	file->private_data=drv ;
      	return 0;

}

static  int 
gpio_vsc_release (struct inode *inode,struct file *file)
{
	gpio_drv_t *drv;
	drv = file->private_data;
	file->private_data = NULL;
	kfree(drv);
	return 0;

}

static  int  
gpio_vsc_read (struct file *file, char *buffer,size_t size, loff_t *offset)
{

	return 0;
	
}

static  int  
gpio_vsc_write (struct file *file, const char *buffer, size_t size, loff_t * offset)
{

	return 0;

}

static int  
gpio_vsc_ioctl (struct inode *inode, struct file *file, uint cmd , ulong arg)
{
	gpio_drv_t *drv;
	gpio_dev_t *dev;
	Gpio_data_t gpin_data;

	//printk("[gpio] : gpio_vsc_ioctl \n");
	drv =file->private_data;
	dev= &gpio_dev; 

      /*Getting data from user space to driver space  */
	if ( copy_from_user(&gpin_data, (void*)arg,sizeof(Gpio_data_t)) )
	{
		printk("[gpio] : copy_from_user failed! \n");
		return -EFAULT;
	}


	switch(cmd)
	{
		  /* GPIO Functions - */ 
		case  GET_GPIO_DIRECTION :
			
			if (drv->setdir)
			{
				if (0!=drv->getdir(dev,(void*)&gpin_data))
				return -1;
			}
                   if (copy_to_user((void*)arg, &gpin_data , sizeof(Gpio_data_t))) return -EFAULT;
 		 	break;
		
		case  SET_GPIO_DIRECTION :
			
			if (drv->setdir)
			{
				if (0!=drv->setdir(dev,(void*)&gpin_data))
				return -1;
			}
			break;
			
		case   READ_GPIO :	
			
			if (drv->setdir)
			{
				if (0!=drv->readgpio(dev,(void*)&gpin_data))
				return -1;
			}
			if (copy_to_user((void*)arg, &gpin_data , sizeof(Gpio_data_t))) return -EFAULT;
 		      break;
			
		case  WRITE_GPIO :
			
			if (drv->writegpio)
			{
				if (0!=drv->writegpio(dev,(void*)&gpin_data))
				return -1;
			}
			break;
		
		 case   GET_INPUT_INTR_STATE :	
			
			if (drv->get_intr_state)
			{
				if (0!=drv->get_intr_state(dev,(void*)&gpin_data))
				return -1;
			}
			if (copy_to_user((void*)arg, &gpin_data , sizeof(Gpio_data_t))) return -EFAULT;
 		      break;
			  
		case   SET_INPUT_INTR_STATE :
	
			if (drv->set_intr_state)
			{
				if (0!=drv->set_intr_state(dev,(void*)&gpin_data)) return -1;

			}
			break;

		case	WAIT_INPUT_INTR :
			
			if (drv->wait_for_intr)
			{
				if(0!=drv->wait_for_intr(dev,(void*)&gpin_data)) return -1;

			}
			break;
			
		 default :
			printk(KERN_ERR "[gpio] : Invalid ioctl(%d) called! \n", cmd);
			return -EINVAL;
		}
		return 0;
}




module_init(gpio_vsc_init);
module_exit(gpio_vsc_exit);


int   
vsc_readgpio( void* gpio_data , void* gpin_data )
{
	volatile uint32_t  Addr;
	volatile uint32_t Value;
	gpio_dev_t *gpio ;
	Gpio_data_t *gpindata ;
	unsigned char PortNumber;
	unsigned char PortPinNumber;
	uint32_t temp32;


	gpio =  gpio_data;  
	gpindata=gpin_data;

	if ( gpindata->PinNum > MAX_GPIOS-1)
	{
		return -1;
	}

	/* Get the Port Number and the pin number within that port */
	PortNumber = GpioPortPinMap[gpindata->PinNum].PortNumber;
	PortPinNumber = GpioPortPinMap[gpindata->PinNum].PortPinNumber;

       /* Get the Address for the pin  */ 
	Addr=(uint32_t) gpio->gpio_addr+GPIOPortBaseOffset[PortNumber]+PortPinNumber*GPIO_PIN_ADDR_UNIT;
	Value=gpio_read_reg(Addr);

      /*Check for the 0th bit  for Gpio pin data to be High or Low*/
	temp32=(Value & 0x00000001);

	/*Passing the vaue to the user space*/  
	if (temp32)
	{
		gpindata->data=1;
	}
	else
	{
		gpindata->data=0;
	}
	return 0;
}

int   
vsc_writegpio( void* gpio_data , void* gpin_data )
{
	
	volatile uint32_t  Addr;
	volatile uint32_t Value;
	gpio_dev_t *gpio ;
	Gpio_data_t *gpindata ;
	unsigned char PortNumber;
	unsigned char PortPinNumber;

	gpio =  gpio_data;  
	gpindata=gpin_data;

	if ( gpindata->PinNum > MAX_GPIOS-1)
	{
		return -1;
	}

	/* Get the Port Number and the pin number within that port */
	PortNumber = GpioPortPinMap[gpindata->PinNum].PortNumber;
	PortPinNumber = GpioPortPinMap[gpindata->PinNum].PortPinNumber;

	/*Get the Address for the pin */ 
	Addr=(uint32_t) gpio->gpio_addr+GPIOPortBaseOffset[PortNumber]+PortPinNumber*GPIO_PIN_ADDR_UNIT;

	/*Set Gpio pin to be  high or low based on the user data */
	if (gpindata->data )
	{
		Value= (uint32_t)OUTPUT_HIGH;
	}
	else
	{
		Value=(uint32_t)OUTPUT_LOW;
	}

	gpio_write_reg(Addr,Value);
	return 0;
	
}


int   
vsc_getdir( void* gpio_data , void* gpin_data )
{
	volatile uint32_t  Addr;
	volatile uint32_t Value;
	gpio_dev_t *gpio ;
	Gpio_data_t *gpindata ;
	unsigned char PortNumber;
	unsigned char PortPinNumber;
	uint32_t temp32;

	gpio =  gpio_data;  
	gpindata=gpin_data;

	if ( gpindata->PinNum > MAX_GPIOS-1)
	{
		return -1;
	}

	/* Get the Port Number and the pin number within that port */
	PortNumber = GpioPortPinMap[gpindata->PinNum].PortNumber;
	PortPinNumber = GpioPortPinMap[gpindata->PinNum].PortPinNumber;

	/*Get the Address for the pin */ 
	Addr=(uint32_t) gpio->gpio_addr+GPIOPortBaseOffset[PortNumber]+PortPinNumber*GPIO_PIN_ADDR_UNIT;
	Value=gpio_read_reg(Addr);

	/*Check for the 1st bit for Direction  to be Input or Output*/
	temp32=(Value & 0x00000002)>>0x01;

      /*Passing the vaue to the user space*/
     /*Return value =1 ,when  Output and  0, when Input */ 
	if (temp32)
	{
		gpindata->data=0;
	}
	else
	{
		gpindata->data=1;
	}
	return 0;
}


int   
vsc_setdir( void* gpio_data , void* gpin_data )
{
	volatile uint32_t  Addr;
	volatile uint32_t Value;
	gpio_dev_t *gpio ;
	Gpio_data_t *gpindata ;
	unsigned char PortNumber;
	unsigned char PortPinNumber;

	gpio =  gpio_data;  
	gpindata=gpin_data;

	if ( gpindata->PinNum > MAX_GPIOS-1)
	{
		return -1;
	}

	/* Get the Port Number and the pin number within that port */
	PortNumber = GpioPortPinMap[gpindata->PinNum].PortNumber;
	PortPinNumber = GpioPortPinMap[gpindata->PinNum].PortPinNumber;

	/*Get the Address for the pin */  
	Addr=(uint32_t) gpio->gpio_addr+GPIOPortBaseOffset[PortNumber]+PortPinNumber*GPIO_PIN_ADDR_UNIT;

	/*Set Gpio pin to be  as Input when 0 is received from user space or Output  when  1 received from user space  */
	if (gpindata->data )
	{
		Value= (uint32_t)OUTPUT_LOW;
	}
	else
	{
		Value=(uint32_t)INPUT_INTR_DISABLE;
	}
	gpio_write_reg(Addr,Value);
	return 0;
}

int set_input_intr_state  (void* gpio_data ,void *gpin_data)
{
	volatile uint32_t  Addr;
	volatile uint32_t Value;
	unsigned char PortNumber;
	unsigned char PortPinNumber;
	gpio_dev_t *gpio ;
	Gpio_data_t *gpindata ;

//	printk(KERN_ALERT " set_input_intr_state  \n ");

	gpio =  gpio_data;  
	gpindata=gpin_data;

	/* Get the Port Number and the pin number within that port */
	PortNumber = GpioPortPinMap[gpindata->PinNum].PortNumber;
	PortPinNumber = GpioPortPinMap[gpindata->PinNum].PortPinNumber;		

	/*Get the Address for the pin */ 
	Addr=(uint32_t) gpio->gpio_addr+GPIOPortBaseOffset[PortNumber]+PortPinNumber*GPIO_PIN_ADDR_UNIT;

	/* Set gpio interrupt 
	0 -No interrupt is enabled 
	1 -Interrupt on rising edge 
	2 -Interrupt on falling edge 
	3 -Interrupt on both falling and rising edge
	*/

	/*Set Gpio pin to be  Interrupt state based on the user data */  
	switch(gpindata->data)
	{
		case 0:
			
			Value= (uint32_t)INPUT_INTR_DISABLE;
			gpio_write_reg(Addr,Value);
			break;

		case 1:
			
			Value= (uint32_t)INPUT_INTR_RISE_EDGE;
			gpio_write_reg(Addr,Value);
			break;

		case 2:
			
			Value= (uint32_t)INPUT_INTR_FALL_EDGE;
			gpio_write_reg(Addr,Value);
			break;

		case 3:
			
			Value= (uint32_t)INPUT_INTR_BOTH_EDGE;
			gpio_write_reg(Addr,Value);
			break;
			
		default:
			printk(KERN_ALERT " Invalid Input Interrupt State  \n ");
			break;
				
	}

	interrupt_pin = 17;
	return 0;
   
}


int get_input_intr_state  (void* gpio_data ,void *gpin_data)
{
	unsigned char PortNumber;
	unsigned char PortPinNumber;
	uint32_t temp32;
	volatile uint32_t  Addr;
	volatile uint32_t Value;
	gpio_dev_t *gpio ;
	Gpio_data_t *gpindata ;

	gpio =  gpio_data;  
	gpindata=gpin_data;

	/* Get the Port Number and the pin number within that port */
	PortNumber = GpioPortPinMap[gpindata->PinNum].PortNumber;
	PortPinNumber = GpioPortPinMap[gpindata->PinNum].PortPinNumber;	

	/*Get the Address for the pin */ 
	Addr=(uint32_t) gpio->gpio_addr+GPIOPortBaseOffset[PortNumber]+PortPinNumber*GPIO_PIN_ADDR_UNIT;
	Value=gpio_read_reg(Addr);


	/* return value 
	0 -No interrupt is enabled 
	1 -Interrupt on rising edge 
	2 -Interrupt on falling edge 
	3 -Interrupt on both falling and rising edge
	*/   

	/*Checking whether Gpio pin Direction is made as Input */
	temp32=(Value & 0x00000002)>>0x01;

	if (temp32)
	{
		/*shifting the last dont care bit and comparing 3-1 bits*/
		if(INPUT_INTR_DISABLE >>0x01 == Value>>0x01)
		{
			gpindata->data=0; 
		}
		if(INPUT_INTR_RISE_EDGE >>0x01 == Value>>0x01)
		{
			gpindata->data=1;
		}
		if(INPUT_INTR_FALL_EDGE >>0x01 == Value>>0x01)
		{
			gpindata->data=2;
		}
		if(INPUT_INTR_BOTH_EDGE>>0x01 == Value>>0x01)
		{
			gpindata->data=3;
		}
	}
	else 
	{
		printk(KERN_ALERT " Direction of the Gpio pin was not as Input  \n ");
		return -1;
	}
	
	return 0;
}

int wait_input_intr (void* gpio_data ,void *gpin_data)
{	
	Gpio_data_t *gpindata ;
	gpindata=gpin_data;

	/* Get the Port Number and the pin number within that port
	   PortNumber = GpioPortPinMap[gpindata->PinNum].PortNumber;
	   PortPinNumber = GpioPortPinMap[gpindata->PinNum].PortPinNumber;	
	*/
	if(wait_event_interruptible(gpio_intr_wq,((interrupt_port == GpioPortPinMap[gpindata->PinNum].PortNumber)  && (interrupt_pin == GpioPortPinMap[gpindata->PinNum].PortPinNumber))))
		return -ERESTARTSYS;

//	printk(KERN_ALERT "interrupt_port: %d interrupt_pin: %d\n", interrupt_port, interrupt_pin);
	interrupt_pin = 17;
	return 0;
	
}

void * get_gpio_dev(void)
{
	return &gpio_dev;
}


EXPORT_SYMBOL_GPL(vsc_setdir);
EXPORT_SYMBOL_GPL(vsc_getdir);
EXPORT_SYMBOL_GPL(vsc_writegpio);
EXPORT_SYMBOL_GPL(vsc_readgpio);
EXPORT_SYMBOL_GPL(get_gpio_dev);
EXPORT_SYMBOL_GPL(set_input_intr_state);
EXPORT_SYMBOL_GPL(wait_input_intr);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("VSC452 GPIO Driver");


	


