/*-------------------------------------------------------------------------
	Copyright (C) 2008-2009	American Megatrends Inc.

	This program is	free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as	published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is	distributed in the hope	that it	will be	useful,
	but WITHOUT ANY	WARRANTY; without even the implied warranty of
	MERCHANTABILITY	or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have	received a copy	of the GNU General Public License
	along with this	program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave,	Cambridge, MA 02139, USA.
-------------------------------------------------------------------------*/
#ifndef	_I2C_HARDWARE_H
#define	_I2C_HARDWARE_H

/* Sensible Defaults */
#define	DEFAULT_TIMEOUT			100	/* Timeout/delay for bus activities */
#define	DEFAULT_RETRIES			3	/* Retries on send/receive */
#define	DEFAULT_BB_RETRIES		5	/* Number of retries to wait for free bus before kicking in recovery code */

#define	DRIVER_NAME			"i2c" 

#define	I2C_REG_SIZE			768	/* bytes */	
#define	I2C_BASE			0xB0200500
#define	INT_OBSERVE_REG			((volatile unsigned long*)0xB02008F8)	/* 0-5 bits of this register holds Interrupt status of (0-5)I2C	bus*/


extern void *i2c_remap_addr ;
extern u8 slave_addr [];

/* I2C Bus virtual address */
#define	VSC_I2C_0_VA_BASE		((unsigned long	*)(((unsigned long)i2c_remap_addr) + 0x000))
#define	VSC_I2C_1_VA_BASE		((unsigned long	*)(((unsigned long)i2c_remap_addr) + 0x080))
#define	VSC_I2C_2_VA_BASE		((unsigned long	*)(((unsigned long)i2c_remap_addr) + 0x100))
#define	VSC_I2C_3_VA_BASE		((unsigned long	*)(((unsigned long)i2c_remap_addr) + 0x180))
#define	VSC_I2C_4_VA_BASE		((unsigned long	*)(((unsigned long)i2c_remap_addr) + 0x200))
#define	VSC_I2C_5_VA_BASE		((unsigned long	*)(((unsigned long)i2c_remap_addr) + 0x280))

/* I2C Register	offset */
#define	VSC_I2CX_DATA			0x000
#define	VSC_I2CX_CNTRL			0x004
#define	VSC_I2CX_BIT_CNTRL		0x008
#define	VSC_I2CX_BIT_STATUS		0x00C
#define	VSC_I2CX_FIFO_CNTRL		0x010
#define	VSC_I2CX_FIFO_STATUS		0x014
#define	VSC_I2CX_FIFO_DATA		0x018

/* Definition for interrupt observe bit	register */
#define	I2C_OBSERVE_INT_0		1<<0
#define	I2C_OBSERVE_INT_1		1<<1
#define	I2C_OBSERVE_INT_2		1<<2
#define	I2C_OBSERVE_INT_3		1<<3
#define	I2C_OBSERVE_INT_4		1<<4
#define	I2C_OBSERVE_INT_5		1<<5

/* Definitions for data	register */
#define	I2C_SET_N_ADDRESS_AS_SLAVE	0x80
#define	I2C_NO_GENERAL_CALL		0x80
#define	I2C_ENABLE_SEND_SLAVE_ADDR	0x80
#define	I2C_SCL_FEQ_100KHZ		0x00
#define	I2C_SCL_FEQ_50KHZ		0x01
#define	I2C_SCL_FEQ_400KHZ		0x08

/* Definitions for control register */
#define	I2C_SET_SLAVE_ADDR		0x00
#define	I2C_SET_CLK_FEQ			0x20
#define	I2C_SET_SEND_SLAVE_ADDR		0x30
#define	I2C_CLEAR_ALL_STATUS		0x80
#define	I2C_ENABLE_INTERFACE		0x40
#define	I2C_ENABLE_INTERRUPT		0x08
#define	I2C_SEND_START			0x04
#define	I2C_SEND_STOP			0x02
#define	I2C_ACKNOWLEDGEMENT		0x01
#define	I2C_CLR_ACKNOWLEDGEMENT		0xFE
#define	I2C_DISABLE_INTERRUPT		0x08

#define	I2C_PENDING_INT_NOT		0x80
#define	I2C_START_STATUS_NOT		0x40
#define	I2C_STOP_STATUS			0x20
#define	I2C_BUS_ERROR			0x10
#define	I2C_LAST_RECEIVED_BIT		0x08
#define	I2C_ADDRESSED_AS_SLAVE		0x04
#define	I2C_LOST_ARBITRATION		0x02
#define	I2C_BUS_BUSY_NOT		0x01

/* Definitions for bit control register	*/
#define	I2C_SERIAL_INTERFACE_SOFT_RESET	0x80
#define	I2C_STOP_DETECTED_INT_ENABLE	0x40
#define	I2C_SERIAL_SCL_HIGH		0x02
#define	I2C_SERIAL_SDA_HIGH		0x01

/* Definitions for bit status register */
#define	I2C_SERIAL_INTERFACE_INT_STATUS	0x80
#define	I2C_STOP_DETECT_INT_STATUS	0x40
#define	I2C_SERIAL_CLK_STATUS		0x02
#define	I2C_SERIAL_DATA_STATUS		0x01
#define	I2C_CLR_STOP_INT		0x01

/* Definitions for FIFO	control	register */
#define	I2C_SLAVE_RX_FIFO_ENABLE	0x80
#define	I2C_CLR_RX_FIFO_PTRS		0x40
#define	I2C_SLAVE_TX_INT_ENABLE		0x02
#define	I2C_RX_FIFO_INT_ENABLE		0x01

/* Definitions for FIFO	status register	*/
#define	I2C_RX_FIFO_FULL		0x08	
#define	I2C_RX_FIFO_EMPTY		0x04
#define	I2C_SLAVE_TX_INT_STATUS		0x02
#define	I2C_RX_FIFO_INT_STATUS		0x01
#define	I2C_CLR_RX_FIFO_FULL_INT	0x01

/* Definitions for bus status */
#define	I2C_SET_BUS_ERROR_BIT		0x08
#define	I2C_SET_BUS_BUSY_BIT		0x04

/* Definitions for control hardware */
#define	I2C_DRV_REC_RESET_SMBUS		0x08
#define	I2C_DRV_REC_CLK_PULSE		0x04
#define	I2C_DRV_REC_RESET_OUT		0x02
#define	I2C_DRV_REC_FORCE_STOP		0x01

#define	WRITE_OPERATION_BIT		0x00
#define	READ_OPERATION_BIT		0x01

/* Definitions for wait_for_int return values */
#define	OP_STATUS_TIMEOUT		0
#define	OP_STATUS_SUCCESS		1
#define	OP_STATUS_NACK			2
#define	OP_STATUS_ARB_LOST		4
#define	OP_STATUS_BUS_ERR		5
#define	OP_STATUS_PENDING		6
#define	OP_STATUS_UNKOWN		7
#define	OP_STATUS_DEV_FOUND		8

/* Definitions for I2C pin states */
#define	HIGH				1
#define	LOW				0

/* Number of consequtive bus recovery attemps before disabling bus */
#define	MAX_ALLOWED_BUS_RECOVERY_ATTEMPTS 20 
#define MAX_BUS_BUSY_CNT		20
#define MAX_FAILURES_PER_DEVICE		20

typedef	unsigned long UINT32;

/* i2c-hardware.c externs */
extern void i2c_init_hardware(int bus);
extern int i2c_vsc_enable_slave(int bus);
extern void i2c_vsc_disable_all_interrupts(int	bus);
extern void i2c_enable_bus(int	bus);
extern void i2c_disable_bus(int bus);
extern void i2c_vsc_disable_interrupt(int bus,	unsigned long mask);
extern void i2c_vsc_enable_interrupt(int bus, unsigned	long mask);
extern void i2c_vsc_set_slave(int bus,	int slave);
extern void i2c_vsc_reset(int bus);
extern unsigned long i2c_vsc_read_reg(unsigned	long addr);
extern void i2c_vsc_write_reg(unsigned	long val, unsigned long	addr);
extern int is_i2c_bus_busy(int	bus);
extern unsigned long get_clk_status(int bus);
extern unsigned long get_sda_status(int bus);
extern unsigned	long i2c_vsc_io_base(int bus_count);
extern int i2c_wait_for_bus_free(int bus);

/* i2c-recovery.c externs */
extern bus_recovery_info_T m_bus_recovery_info[];
extern int vsc_get_recovery_info(int bus, bus_recovery_info_T* info);
extern int vsc_set_recovery_info(int bus, bus_recovery_info_T* info);
extern int vsc_i2c_bus_recovery(struct i2c_adapter *i2c_adap);
extern int vsc_is_i2c_bus_hung(struct i2c_adapter *i2c_adap);
extern void reset_mux_via_gpio(int bus);


/* i2c-transfer.c externs */
extern int vsc_master_xfer(struct i2c_adapter *i2c_adap, struct i2c_msg *msgs,	int num);
extern int vsc_slave_xfer(struct i2c_adapter *i2c_adap, char *buf, int	num);

/* i2c-interrupt.c externs */
extern irqreturn_t i2c_handler(int this_irq, void *dev_id);

/* i2c-module.c externs */
void i2c_vsc_exit(void);

#endif

