/*
 * Carsten Langgaard, carstenl@mips.com
 * Copyright (C) 1999,2000 MIPS Technologies, Inc.  All rights reserved.
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * PROM library functions for acquiring/using memory descriptors given to
 * us from the YAMON.
 * 
 */
 
/* 
 *
 *  Modifications done to work with U-boot
 *  Copyright (C) 2008 American Megatrend, Inc.  All rights reserved.
 *
 */
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/bootmem.h>
#include <linux/string.h>

#include <asm/bootinfo.h>
#include <asm/page.h>

#include <asm/mips-boards/prom.h>
#include "../../../../projdef.h"




/* References to section boundaries */
extern char _end;

#define PFN_ALIGN(x)    (((unsigned long)(x) + (PAGE_SIZE - 1)) & PAGE_MASK)


unsigned long __init prom_getmem_size(void)
{
	char *memsize_str;
	unsigned int memsize;
	char cmdline[CL_SIZE], *ptr;

	/* Check the command line first for a memsize directive */
	strcpy(cmdline, arcs_cmdline);
	ptr = strstr(cmdline, "memsize=");
	if (ptr && (ptr != cmdline) && (*(ptr - 1) != ' '))
		ptr = strstr(ptr, " memsize=");

	if (ptr) {
		memsize = memparse(ptr + 8, &ptr);
	}
	else {
		/* otherwise look in the environment */
		memsize_str = prom_getenv("memsize");
		if (!memsize_str) {
			printk("memsize not set in boot prom, set to default (0x%x)\n", CFG_PROJ_MEMORY_SIZE);
			memsize = CFG_PROJ_MEMORY_SIZE;
		} else {
			memsize = simple_strtol(memsize_str, NULL, 0);
		}
	}
	
	return memsize;

}

#if 0 
unsigned long __init prom_getmem_start(void)
{
	char *memstart_str;
	unsigned int memstart;


	/* Look in the environment */
	memstart_str = prom_getenv("memstart");
	if (!memstart_str) 
	{
		printk("memstart is not set in boot prom, set to default (0x%x)\n", CFG_PROJ_MEMORY_START);
		memstart = CFG_PROJ_MEMORY_START;
	} else 
	{
		memstart = simple_strtol(memstart_str, NULL, 0);
	}

	
	return memstart;

}
#endif



void __init prom_meminit(void)
{
	unsigned long memstart;
	unsigned long memsize;
	

	/* 
	    Memory Type can be 
		BOOT_MEM_RAM
		BOOT_MEM_ROM_DATA
		BOOT_MEM_RESERVED
	*/
	
	/* Add RAM to memory region */
//	memstart = prom_getmem_start();
	memstart = 0;
	memsize = prom_getmem_size();

	if (memsize)
		add_memory_region(memstart,memsize,BOOT_MEM_RAM);


	
}

void __init prom_free_prom_memory(void)
{
        unsigned long addr;
        int i;

        for (i = 0; i < boot_mem_map.nr_map; i++) {
                if (boot_mem_map.map[i].type != BOOT_MEM_ROM_DATA)
                        continue;

                addr = boot_mem_map.map[i].addr;
                free_init_pages("prom memory",
                                addr, addr + boot_mem_map.map[i].size);
        }
}
