/* ------------------------------------------------------------------------- */
/* Copyright (C) 2009 American Megatrends Inc.
 *
 * ECC Device Driver                 					     				 */
/* ------------------------------------------------------------------------- */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/sched.h>
#include <linux/kernel.h>
#include <linux/ide.h>
#include <linux/sysfs.h>
#include <linux/delay.h>
#include <linux/notifier.h>
#include <linux/reboot.h>

#include "vsc_ecc.h"

#define WRITE_OPER_SUPPORT	0	//Enable this macro, only when write operation is needed by sysfs files

#define dbgprint        	if(Debug_Enable) printk

#define FHR ((volatile unsigned long*)0xB02008F4) // Force Hardware Reset 
//#define FORCE_SYSTEM_RESET     0x0080 
//#define FORCE_CPU_RESET        0x0040


/******* Structures used for ECC *******/

static struct sysfs_ops ecc_sysfs_ops = {
	.show = show_ecc_reg,
	.store = store_ecc_reg,
};

static struct kobj_type ecc_kobj_ktype = {
	.sysfs_ops = &ecc_sysfs_ops,
};

static struct notifier_block vsc_ecc_notifier =
{
   .notifier_call = vsc_ecc_notify_sys,
};

struct class_private {
    struct kset class_subsys;
    struct klist class_devices;
    struct list_head class_interfaces;
    struct kset class_dirs;
    struct mutex class_mutex;
    struct class *class;
};

static struct file_operations ecc_fops = {
	owner:          THIS_MODULE,
	read:           NULL,
    write:          NULL,
    ioctl:          vsc_ecc_ioctl,
    open:           NULL,
    release:        NULL,
};

struct ecc_file_attrib
{
	char *atname;
	unsigned long counter;
	struct attribute attr;
	struct ecc_file_attrib *next_attr;
};

/******* Variable Declaration *********/
static short Debug_Enable=0;
static const int ECC_DEVNUM = 82;
static const char ECC_DEVNAME[] = "ECC";
extern struct class *get_hwmon_class(void);
static struct kobject *ecc_kobj;
static struct class *parent_cls;
struct ecc_file_attrib *head = NULL, *end = NULL, *tmp_ptr = NULL, *search_ptr = NULL;
short ecc_flag = SLEEP;
short reset_flag = FALSE;
unsigned long g_arg = 0,g_incr = 0;
DECLARE_WAIT_QUEUE_HEAD(ecc_wait_queue);

/******** Method Definitions **********/

/*************************************************************************************************
 *                      Driver Read Register Call
 *
 * @ Description:
 * 	Function used to read the contents of the specific register address
 *
 * @ Arguments:
 * 	addr -	Address from which data has to be read
 *
 * @ Returns:
 * 	Data on the specific register address
 *************************************************************************************************/
unsigned long vsc_ecc_read_reg(unsigned long addr)
{
    return (*(volatile uint32_t*)addr);
}

/*************************************************************************************************
 *                      Driver Write Register Call
 *
 * @ Description:
 * 	Function used to write data onto the specific register address
 *
 * @ Arguments:
 * 	val  	  -	Value which has to be written onto the specific address
 * 	operation - 	Operation which has to be carried out before writing the value
 * 	addr	  -	Address on which data has to be written
 *************************************************************************************************/
void vsc_ecc_write_reg(unsigned long val, int operation, unsigned long addr)
{
	switch(operation)
	{
		case NONE :	*((volatile uint32_t*)addr) = val;
					break;
		case AND  :	*((volatile uint32_t*)addr) = (vsc_ecc_read_reg((volatile unsigned long)addr) & val);
					break;
		case OR	  :	*((volatile uint32_t*)addr) = (vsc_ecc_read_reg((volatile unsigned long)addr) | val);
					break;
		default	  :	printk(KERN_ALERT "ECC: Unknown operation under writing to register\n");
					break;
	}
    return;
}

/*************************************************************************************************
 *                      Driver Show Call
 *
 * @ Description:
 * 	Function which gets called when we invoke read operation on the sysfs files created inside
 * /sys/class/hwmon/ecc folder.
 *************************************************************************************************/
static ssize_t show_ecc_reg(struct kobject *kobj, struct attribute *attr,char *chr)
{
    dbgprint(KERN_ALERT "Name got inside show function is %s\n", attr->name);
	if(head != NULL)
	{
		search_ptr = head;
		/* Searching over the linked list to find the desired file counter value */
		while(search_ptr->next_attr != NULL)
		{
			if(strcmp(search_ptr->atname, attr->name) == 0)
			{
        		dbgprint(KERN_ALERT "Name matched inside show function \n");
				return sprintf( chr, "%lx\n", search_ptr->counter);
			}
			else
			{
				search_ptr = search_ptr->next_attr;
			}
		}
		/*If only one node is present or else last node encounters */
		if(strcmp(search_ptr->atname, attr->name) == 0)
		{
			dbgprint(KERN_ALERT "Name got at head node(or)last node is %s\n", attr->name);
			return sprintf( chr, "%lx\n", search_ptr->counter);
		}
	}
	/* Dummy return statement to suppress compiling issues */
    return sprintf( chr, "%d\n" , -1);
}

/*************************************************************************************************
 *                      Driver Store Call
 *
 * @ Description:
 *      Function which gets called when we invoke write operation on the sysfs files created inside
 * /sys/class/hwmon/ecc folder.
 **************************************************************************************************/
static ssize_t store_ecc_reg(struct kobject *kobj,struct attribute *attr,const char *chr, size_t size)
{

    dbgprint(KERN_ALERT "Name got inside store function is %s\n", attr->name);

#if (WRITE_OPER_SUPPORT == 1)
    unsigned long val = 0;
	if(head != NULL)
	{
		search_ptr = head;
		while(search_ptr->next_attr != NULL)
		{
			if(strcmp(search_ptr->atname,attr->name) == 0)
			{
        		dbgprint(KERN_ALERT "Name matched inside store function \n");
				sscanf(chr, "%lx", &val);
				search_ptr->counter = val;
				return size;
			}
			else
			{
				search_ptr = search_ptr->next_attr;
			}
		}
		/* If only one node is present or else last node encounters */
		if(strcmp(search_ptr->atname, attr->name) == 0)
		{
			dbgprint(KERN_ALERT "Name got at head(or)last node is %s\n", attr->name);
			sscanf(chr, "%lx", &val);
			search_ptr->counter = val;
			return size;
		}
	}
#else
	printk(KERN_ALERT "ECC: Sorry, write operation not supported for %s sysfs file\n", attr->name);
#endif

	return size;
}

/*************************************************************************************************
 * 			Driver Reboot Notifier
 ************************************************************************************************/
static int vsc_ecc_notify_sys(struct notifier_block *this, unsigned long code, void *unused)
{
	/* Reset error injection bit */
	vsc_ecc_write_reg( RESET_ERROR_INJECTION, NONE, ECC_CNTRL_REG);

	/* Setting interrupt mask to mask all errors */
    vsc_ecc_write_reg( MASK_ALL_INTERRUPT, OR, ECC_INTERRUPT_MASK_REG);

    /* Disable ECC interrupts for all agents */
    vsc_ecc_write_reg( DISABLE_ECC_INTERRUPT_FOR_ALL, OR, ECC_CNTRL_2_REG);

	return NOTIFY_DONE;
}

/*************************************************************************************************
 * 			Driver Init Module Call
 ************************************************************************************************/
static int init_ecc_module(void)
{
	int err,retval = 0;
	printk(KERN_ALERT "Loading ECC Driver\n");
	if ((err = register_chrdev(ECC_DEVNUM, ECC_DEVNAME, &ecc_fops)) < 0)
    {
    	printk(KERN_ERR "ECC: Failed to register device (err %d)\n", err);
        retval = -ENODEV;
        goto onerror;
	}
	if ((err = request_irq(ECC_IRQ_NUM, ecc_int_handler, IRQF_DISABLED, "ECC", 0)) < 0)
    {
    	printk(KERN_ERR "ECC: Failed to request irq %d (err %d)\n", ECC_IRQ_NUM, err);
        retval = -EBUSY;
		goto onerror;
	}
	/* Get parent class instance */
	parent_cls = get_hwmon_class();

	/* Create /sys/class/hwmon/ecc sysfs directory */
	ecc_kobj = kobject_create_and_add ( "ecc", &parent_cls->p->class_subsys.kobj );
	if(ecc_kobj == NULL)
	{
		printk(KERN_ERR "ECC: Can't create a ECC sysfs directory\n");
		retval = -ENOMEM;
		goto onerror;
	}
	/* Assign store and show action on the sysfs files */
	ecc_kobj->ktype = &ecc_kobj_ktype;
	kobject_uevent(ecc_kobj, KOBJ_ADD);

	err = register_reboot_notifier(&vsc_ecc_notifier);
	if(err < 0)
	{
		printk(KERN_ALERT "ECC: Reboot notifier registration failed\n");
		retval = -EBUSY;
		goto onerror;
	}
	err = kernel_thread(vsc_ecc_thread, (void *) NULL, 0);
	if(err < 0)
	{
		printk(KERN_ALERT "ECC: Thread creation failed\n");
		retval = -EBUSY;
		goto onerror;
	}

	printk(KERN_ALERT "ECC Driver Loaded Successfully\n");
	return retval;

onerror:
	printk(KERN_ALERT "ECC: Loading failed\n");
	cleanup_ecc_module();
	return retval;
}

/*************************************************************************************************
 *                      Driver Interrupt Handler Call
 ************************************************************************************************/
static irqreturn_t ecc_int_handler ( int irq, void *dev_id )
{
	irqreturn_t retirq = IRQ_NONE;
	volatile unsigned long ior,intstat,erraddr,errcount;
//	void (*reset_sp)(void);

	dbgprint(KERN_ALERT "Inside ECC Interrupt Handler\n");
	ior = vsc_ecc_read_reg( INT_OBSERVE_REG );

	/* Check whether interrupt is exactly for us */
	if(ior & ECC_INTERRUPT_BIT )
	{
		dbgprint(KERN_ALERT "Got interrupt for ECC Interface\n");

		intstat = vsc_ecc_read_reg( ECC_INTERRUPT_STATUS_REG );

		dbgprint(KERN_ALERT "Interrupt status register for ECC has value:%lx\n", intstat);

		if(intstat & CORRECTABLE_ERR_STATUS_BIT)
		{
			dbgprint(KERN_ALERT "Correctable error occured\n");

			vsc_ecc_write_reg(RESET_ERROR_INJECTION, NONE, ECC_CNTRL_REG);

			/* Get correctable error address */
			erraddr = vsc_ecc_read_reg( ECC_CORRECTABLE_ERR_ADDR_REG );
			/* Get correctable error count */
			errcount = vsc_ecc_read_reg( ECC_CORRECTABLE_ERR_COUNT_REG );

			/* Update global variable which is to be used inside thread */
			ecc_flag = WAKEUP;
			g_arg = (unsigned long)(0x80000000 + (erraddr * 4));

			/* Filtering only rising edge strobe for correctable error count */
			g_incr = (errcount & MASK_CORRECTABLE_ERROR);

			/* Clear the correctable error counter */
			vsc_ecc_write_reg( CLEAR_CORRECTABLE_ERR_COUNTER, AND, ECC_CORRECTABLE_ERR_COUNT_REG);

			/* Wake up the thread to create (or) update file */
			wake_up_interruptible(&ecc_wait_queue);
		}
		else if(intstat & UNCORRECTABLE_ERR_STATUS_BIT)
		{
			vsc_ecc_write_reg(RESET_ERROR_INJECTION, NONE, ECC_CNTRL_REG);

			printk(KERN_ALERT "Uncorrectable error occured, SP going to reboot\n");

			dbgprint(KERN_ALERT "Uncorrectable error occured\n");
			
			/* Update KCS KCD register to indicate Uncorrectable error */
			vsc_ecc_write_reg(CLEAR_IBF_OBF_FLAG, NONE, KCS2_STATUS_REG);
			vsc_ecc_write_reg(SP_SELF_REPAIR, NONE, KCS2_DATA_REG);
			vsc_ecc_write_reg(SET_OBF_FLAG, NONE, KCS2_STATUS_REG);

			/* Update POS register to indicate Uncorrectable error */
			vsc_ecc_write_reg(UNCORRECTABLE_ECC_PANIC, NONE, VSC_POS_REG);
		
			/* Update the SRAM address to indicate OEM_RESET_PANIC occured */
			vsc_ecc_write_reg(OEM_RESET_PANIC, NONE, SRAM_INTERFACE_REG);
	
			/* On reset from uboot . We dont jump to reset vector location 0xbfc00000 , instead make the CPU to 
			 send SI_Reset using Force Hardware Reset Register , which result in  making reset vector in flash ( BEV = 1) */ 
			*(FHR) = FORCE_SYSTEM_RESET; 
		#if 0
			/* Reset SP hardware immediately */
			reset_sp = (void *) 0xbfc00000;
			(*reset_sp)();
		#endif 
		}
		else if(intstat & OUT_OF_RANGE_ADDR_STATUS_BIT)
		{
			/* No need to handle out of range address errors */
			dbgprint(KERN_ALERT "Out of range address error occured\n");
			
			/* Clear the out of range error counter */
			vsc_ecc_write_reg( CLEAR_OUT_OF_RANGE_ERR_COUNTER, AND, ECC_UNCORRECTABLE_ERR_INFO_REG);
		}
		else
		{
			printk(KERN_ALERT "ECC: Unknown interrupt condition inside interrupt handler\n");
		}
		/* Clearing the interrupt by writting back 1 onto the corresponding error bit */
		if( (intstat & CORRECTABLE_ERR_STATUS_BIT) && (intstat & UNCORRECTABLE_ERR_STATUS_BIT) )
		{
			vsc_ecc_write_reg( (OUT_OF_RANGE_ADDR_STATUS_BIT | CORRECTABLE_ERR_STATUS_BIT | UNCORRECTABLE_ERR_STATUS_BIT), OR, ECC_INTERRUPT_STATUS_REG);
		}
		else if( intstat & CORRECTABLE_ERR_STATUS_BIT )
			vsc_ecc_write_reg( (OUT_OF_RANGE_ADDR_STATUS_BIT | CORRECTABLE_ERR_STATUS_BIT), OR, ECC_INTERRUPT_STATUS_REG);
		else if( intstat & UNCORRECTABLE_ERR_STATUS_BIT )
			vsc_ecc_write_reg( (OUT_OF_RANGE_ADDR_STATUS_BIT | UNCORRECTABLE_ERR_STATUS_BIT), OR, ECC_INTERRUPT_STATUS_REG);
		else
			printk(KERN_ALERT " ECC: Some other condition met while clearing interrupt status\n");

		retirq = IRQ_HANDLED;
	}
	return retirq;
}

/*************************************************************************************************
 *                      Driver IOCTL Call
 ************************************************************************************************/
static int vsc_ecc_ioctl(struct inode *inode, struct file *file,unsigned int cmd, unsigned long arg)
{
	int err=0;
	dbgprint(KERN_ALERT "Inside ECC IOCTL call with cmd:0x%02x and with arg:0x%lx\n",cmd,arg);
	switch(cmd)
	{
		case INJECT_ERROR: 	dbgprint(KERN_ALERT "ECC Testing: Going to inject error onto ECC\n");
							/* This one will inject errors to test ECC driver functionality */
							if(arg == UNCORRECTABLE_ERROR)
                            {
                                dbgprint(KERN_ALERT "Going to inject uncorrectable error by writing 0x00FF1305\n");
                                vsc_ecc_write_reg(INJECT_UNCORRECTABLE_ERROR, NONE, ECC_CNTRL_REG);
                            }
                            else if(arg == CORRECTABLE_ERROR)
                            {
                                dbgprint(KERN_ALERT "Going to inject correctable error by writing 0x00FFFF05\n");
                                vsc_ecc_write_reg(INJECT_CORRECTABLE_ERROR, NONE, ECC_CNTRL_REG);
                            }
							break;

		case CREATE_FILE:	dbgprint(KERN_ALERT "ECC Testing: Creating %lx file on /sys/class/hwmon/ecc/\n",arg);
							/* Dummy call to create a file */
							err = vsc_ecc_create_file(arg,1);
							if(err != 0)
								printk(KERN_ALERT "ECC Testing: Creating %lx file failed\n", arg);
							break;

		default:	   		printk(KERN_ALERT "ECC Testing: Unknown command received (cmd = %d)\n",cmd);
							break;

	}
	return 0;
}

/*************************************************************************************************
 *                      Function Call to create SYSFS files
 *
 *   @ Description:
 * 	Function call which gets invoked from thread (or) IOCTL for creating
 *   SYSFS files under /sys/class/hwmon/ecc directory
 *
 *   @ Arguments:
 *   arg  -	Name of the sysfs file which has to be prefixed with '0x'
 *   incr -	Increament number that has to be added to the count
 *
 *   @ Return:
 *   0	-	Success
 *   -1	-	Failure
 ************************************************************************************************/
static int vsc_ecc_create_file(unsigned long arg,int incr)
{
	int err=0;
	char *tmp_name;

	/* Creating a temporary pointer to hold the file name */
	tmp_name = (char *)kmalloc(ATTRIB_NAME_SIZE,GFP_KERNEL);
	sprintf(tmp_name,"0x%lx",arg);
    dbgprint(KERN_ALERT "Name got as %s\n", tmp_name);

	/* Checking if file is already created
	 * (sysfs doesn't check for multiple files with same name)
	 * So the following checks are mandatory
 	 * */
	if(head != NULL)
    {
		search_ptr = head;
		while(search_ptr->next_attr != NULL)
	    {
			/* Check whether the file is already present */
			if(strcmp(search_ptr->atname, tmp_name) == 0)
			{
				dbgprint(KERN_ALERT "Name matched inside file creation function \n");
				/* File found,so updating its counter */
				search_ptr->counter = search_ptr->counter + incr;
				dbgprint(KERN_ALERT "Counter increamented for %s file\n",search_ptr->attr.name);
				/* Free the temporary pointer */
				kfree(tmp_name);
                return 0;
			}
			else
			{
				search_ptr = search_ptr->next_attr;
			}
		}
		/* If only one node is present */
		if(strcmp(search_ptr->atname, tmp_name) == 0)
        {
			dbgprint(KERN_ALERT "Name matched inside file creation function \n");
			/* File found,so updating its counter */
			search_ptr->counter = search_ptr->counter + incr;
			dbgprint(KERN_ALERT "Counter increamented for %s file\n",search_ptr->attr.name);
			/* Free the temporary pointer */
			kfree(tmp_name);
           	return 0;
        }
	}
	/* Free the temporary pointer */
	kfree(tmp_name);

	/* If file is not present, create it */
	tmp_ptr = (struct ecc_file_attrib *)kmalloc(sizeof(struct ecc_file_attrib),GFP_KERNEL);
	if(tmp_ptr == NULL)
	{
		printk(KERN_ERR "Memory allocation error while creating file\n");
		return -1;
	}
	tmp_ptr->atname = (char *)kmalloc(ATTRIB_NAME_SIZE,GFP_KERNEL);
	if(tmp_ptr->atname == NULL)
	{
		printk(KERN_ERR "Memory allocation error while creating file\n");
		/* Release allocated memory */
		kfree(tmp_ptr);
		return -1;
	}
	sprintf(tmp_ptr->atname,"0x%lx",arg);
	tmp_ptr->counter = incr;
	tmp_ptr->attr.name = tmp_ptr->atname;
    tmp_ptr->attr.owner = THIS_MODULE;
    tmp_ptr->attr.mode = S_IRUGO | S_IWUSR;
    tmp_ptr->next_attr = NULL;

	/* Create sysfs file */
	err = sysfs_create_file(ecc_kobj,&tmp_ptr->attr);
	if(err < 0)
	{
		printk(KERN_ERR "Creating %lx sysfs file for ECC failed\n",arg);
		/* Release allocated memory */
		kfree(tmp_ptr);
		kfree(tmp_ptr->atname);
		return -1;
	}

	dbgprint(KERN_ALERT "ECC : Created %lx file for ECC\n",arg);

	/* Update linked list */
	if(head == NULL)
	{
		head = tmp_ptr;
	}
	else
	{
		end->next_attr = tmp_ptr;
	}
	end = tmp_ptr;

	dbgprint(KERN_ALERT "Head name is %s\n",head->attr.name);
	dbgprint(KERN_ALERT "Current Ptr name is %s\n",tmp_ptr->attr.name);
	dbgprint(KERN_ALERT "End name is %s\n",end->attr.name);

	return 0;
}

/*************************************************************************************************
 *                      Driver Thread Call Function
 *
 * @ Description:
 * 	Function which gets called by thread during driver initialisation phase itself
 *
 * @ Argument:
 * 	ecc_arg - 	Dummy argument (Not used)
 *************************************************************************************************/
int vsc_ecc_thread(void * ecc_arg)
{
	/* This call will make the thread available under ps -ax */
	daemonize("ECCthread");
	while(1)
	{
		/* Wait here,till wake_up call is invoked inside interrupt handler */
		wait_event_interruptible(ecc_wait_queue,(ecc_flag == WAKEUP));

		/* Check whether module is unloaded or not */
		if(reset_flag == TRUE)
		{
			break;
		}

		/* Change back ecc_flag status to sleep */
		ecc_flag = SLEEP;

		/* Create SYSFS file using the global variable values */
		if(vsc_ecc_create_file(g_arg,g_incr) < 0)
			printk(KERN_ERR "Error creating file under ECC thread\n");
	}
	dbgprint(KERN_ALERT "ECC thread stopped\n");
	return 0;
}

/*************************************************************************************************
 *                      Register Print Call
 * @ Description
 * 	Function used to print the contents of ECC register
 ************************************************************************************************/
void print_ecc_reg(void)
{
	printk(KERN_ALERT "*********************ECC Register values************************\n");
	printk(KERN_ALERT "Correctable Error reg(0xB0200F30)        : 0x%08lx\n",vsc_ecc_read_reg(ECC_CORRECTABLE_ERR_COUNT_REG));
	printk(KERN_ALERT "Error Syndrome reg(0xB0200F34)           : 0x%08lx\n",vsc_ecc_read_reg(ECC_ERR_SYNDROME_REG));
	printk(KERN_ALERT "Uncorrectable Error Addr reg(0xB0200F38) : 0x%08lx\n",vsc_ecc_read_reg(ECC_UNCORRECTABLE_ERR_ADDR_REG));
	printk(KERN_ALERT "Correctable Error Addr reg(0xB0200F3C)   : 0x%08lx\n",vsc_ecc_read_reg(ECC_CORRECTABLE_ERR_ADDR_REG));
	printk(KERN_ALERT "Out of Range Addr reg(0xB0200F40)        : 0x%08lx\n",vsc_ecc_read_reg(ECC_OUT_OF_RANGE_ADDR_REG));
	printk(KERN_ALERT "Uncorrectable Error Info reg(0xB0200F44) : 0x%08lx\n",vsc_ecc_read_reg(ECC_UNCORRECTABLE_ERR_INFO_REG));
	printk(KERN_ALERT "Interrupt Status reg(0xB0200F48)         : 0x%08lx\n",vsc_ecc_read_reg(ECC_INTERRUPT_STATUS_REG));
	printk(KERN_ALERT "Interrupt Mast reg(0xB0200F4C)           : 0x%08lx\n",vsc_ecc_read_reg(ECC_INTERRUPT_MASK_REG));
	printk(KERN_ALERT "ECC Control reg(0xB0200F5C)              : 0x%08lx\n",vsc_ecc_read_reg(ECC_CNTRL_REG));
	printk(KERN_ALERT "Priority reg(0xB0200F60)                 : 0x%08lx\n",vsc_ecc_read_reg(ECC_INTERRUPT_PRIORITY_REG));
	printk(KERN_ALERT "ECC Control reg 2(0xB0200F68)            : 0x%08lx\n",vsc_ecc_read_reg(ECC_CNTRL_2_REG));
	printk(KERN_ALERT "*******************************************************************\n");
}

/*************************************************************************************************
 *                      Driver Remove Module Call
 ************************************************************************************************/
static void cleanup_ecc_module(void)
{
	printk(KERN_ALERT "Unloading ECC Driver...\n");

	/* First, stop the thread by breaking its while loop */
	ecc_flag = WAKEUP;
	reset_flag = TRUE;
	wake_up_interruptible(&ecc_wait_queue);

	/* Setting interrupt mask to mask all errors */
	vsc_ecc_write_reg( MASK_ALL_INTERRUPT, OR, ECC_INTERRUPT_MASK_REG);
	
	/* Disable ECC interrupts for all agents */
	vsc_ecc_write_reg( DISABLE_ECC_INTERRUPT_FOR_ALL, OR, ECC_CNTRL_2_REG);

	free_irq(ECC_IRQ_NUM,0);
	unregister_reboot_notifier(&vsc_ecc_notifier);
	unregister_chrdev(ECC_DEVNUM, ECC_DEVNAME);

	/* Remove /sys/class/hwmon/ecc sysfs directory */
	kobject_put (ecc_kobj);

	/* Delay required as thread gets unloaded */
	msleep(1); //1 millisecond delay to let the thread kill itself

	/* Release all allocated memory */
	if(head != NULL)
	{
		while(head->next_attr != NULL)
		{
			tmp_ptr = head->next_attr;
			dbgprint(KERN_ALERT "Free called on link with name %s\n", head->atname);
			kfree(head->atname);
			kfree(head);
			head = tmp_ptr;
		}
		dbgprint(KERN_ALERT "Free called on last link with name %s\n", head->atname);
		kfree(head->atname);
		kfree(head);
	}

	printk(KERN_ALERT "ECC Driver Unloaded\n");
	return;
}

module_init (init_ecc_module);
module_exit (cleanup_ecc_module);

MODULE_AUTHOR("American Megatrends Inc.");
MODULE_DESCRIPTION("ECC Driver Module.");
MODULE_LICENSE("GPL");
