/* 
 * (C) Copyright 2003
 * Thomas.Lange@corelatus.se
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <asm/mipsregs.h>
#include <asm/io.h>
#include <vsc452/vsc452regs.h>

#define MAX_GPIO_PORTS          5
#define MAX_GPIOS		80
#define GPIO_PORT0		0x10200a00
#define GPIO_OFFSET		4
#define INPUT_INTR_DISABLE	0x00000003
#define GPIO(port, number)	GPIO_PORT0 + (port *0x40) + (number * GPIO_OFFSET)
#define PHY_RESET		GPIO(1,11)

#define TIMER_4_INITIAL_COUNT   0x10200180
#define TIMER_4_CURRENT_COUNT   0x10200184
#define TIMER_4_MODE_CONTROL    0x10200190
#define TIMER_4_INT_STATUS      0x10200194

void UbootWatchdogEnable(void);
void UbootWatchdogDisable(void);
#define write32(addr, val)	writel(val, 0xa0000000 + (addr))
#define read32(addr)		readl(0xa0000000 + (addr))


long int initdram(int board_type)
{
	/* Sdram is setup by assembler code */
	/* If memory could be changed, we should return the true value here */
	return CFG_SDRAM_LEN;
}


/* Initialise GPIO pins */
unsigned int
initialize_gpio (void)
{
	int i;
	uint32_t GPIO_interrupt_status[MAX_GPIO_PORTS] = {0x1020080C, 0x1020081C, 0x1020082C, 0x1020083C, 0x1020084C}; /* These are register offset addresses */

	for(i=0; i < MAX_GPIOS; i++)
	{
		write32((GPIO_PORT0+(i*GPIO_OFFSET)), INPUT_INTR_DISABLE);
	}

	/* Clear all pending GPIO interrupt */
	for(i=0; i < MAX_GPIO_PORTS; i++)
 	{
 		write32(GPIO_interrupt_status[i], 0xFFFF);
	}

	return 0;
}


int checkboard (void)
{
	int i;
	write32(PHY_RESET, 0); /* Toggle the reset line of the PHY */
	udelay(10);
	write32(PHY_RESET, 1);

	/* Clear all pending I2C interrupt */
	for (i = 0; i < 6; i++)
	{
		*((unsigned long *)(0xB0200504 + i*0x80)) = 1<<7;
	}
	
	return 0;
}

short 
ReadJumperConfig(void)
{
	/* Return Path ID */	
	return -1;	
}

/*
 * Do watchdog interrupt disabling - its needed in case of cold reset
 * We are supporting 3,4,7 and 8 Timers.
 *
 */
unsigned int
disable_watchdog (void)
{
       /* Timer 3 */
       *((unsigned long *)0xB0200170) = 0;
       *((unsigned long *)0xB0200174) = 0;

       /* Timer 4 */
       *((unsigned long *)0xB0200190) = 0;
       *((unsigned long *)0xB0200194) = 0;

       /* Timer 7 */
       *((unsigned long *)0xB02001F0) = 0;
       *((unsigned long *)0xB02001F4) = 0;

       /* Timer 8 */
       *((unsigned long *)0xB0200210) = 0;
       *((unsigned long *)0xB0200214) = 0;

       return 0;

}

/*
 * Using timer 4 as Uboot Watchdog timer  and this timer  shall be reused again in Kernel, as SP Kernel Watchdog Timer with timer 4
 *   
 */

void
UbootWatchdogEnable(void)
{
        /*0x773593ff is equal to 20 seconds at 20MHz.*/
        write32(TIMER_4_INITIAL_COUNT, 0x773593FF);

        /*Timer Interrupt Eanble.*/
        write32(TIMER_4_INT_STATUS, 0x01);

        /*bit0: Enable timer, bit1: Enable watchdog timer mode.*/
        write32(TIMER_4_MODE_CONTROL, 0x03);

        return;
}

void
UbootWatchdogDisable(void)
{
        write32(TIMER_4_INITIAL_COUNT, 0x00);
        write32(TIMER_4_MODE_CONTROL, 0x00);
        write32(TIMER_4_INT_STATUS, 0x00);

        return;
}


#ifdef CFG_READ_MAC_FROM_FRU_SUPPORT

/* 
 * Read Platform from FRU 
 * 
 * Set the platformid as Absolute if string  "Absolut" or "SB_XVIII" ( "PLAT:SB_XXXX;" ) found in FRU.  
 * 
 */
int
parsePlatformidAndSet (unsigned char *FruBuf, char * env_platformid)
{
	
	int i ;
	char AbsolutStr[8];
	char CarnegieStr[9];
	char ReferenceStr[10];
	char VespaStr[6];
	char platStr[6];
	char TempBuf[9]= {0};
	
	memcpy(AbsolutStr, "Absolut", 7);
	memcpy(CarnegieStr, "Carnegie", 8);
	memcpy(ReferenceStr, "Reference", 9);
	memcpy(VespaStr, "Vespa", 5);
	memcpy(platStr, "PLAT:", 5);
	
	/* Search FruBuf for platfromid string */
	for (i=0; i<256-6; i++)
	{
		/* Search platfrom id string "Carnegie"  */
		if (0 == memcmp(FruBuf+i, CarnegieStr, 8))
		{
			strcpy (env_platformid, "Carnegie");	
			break;
		}
		
		/* Search platfrom id string "Absolut"  */
		if (0 == memcmp(FruBuf+i, AbsolutStr, 7))
		{
			strcpy(env_platformid, "Absolut");
			break;
		}

		/* Search platfrom id string "Vespa"  */
		if (0 == memcmp(FruBuf+i, VespaStr, 5))
		{
			strcpy(env_platformid, "Vespa");	
			break;
		}

		/* Search platfrom id string "Reference" */
		if (0 == memcmp(FruBuf+i, ReferenceStr, 9))
		{
			strcpy(env_platformid, "Reference");
			break;
		} 

		/* Search platfrom id string "PLAT:" */
		if (0 == memcmp(FruBuf+i, platStr, 5))
		{
			/* 
		        *If found, copy ASCII platrom "PLAT:SB_XXXX;"  . Store the value of "SB_XXXX"  in TempBuf array and append "\0"as  
			*                              ^-- starts here 
                       *      to last element of array to make it an string ( for string comparison ) 
			*/
			int j=0; 
			/*
			*	"PLAT:SB_XXXX" 
			*             ^--j starts here [ *(FruBuf+i+5+j) ]
			*
			*/
			for (j=0; j<=8; j++)  
			{	
				/*Store the values TempBuf in untill we found the ascii ";"  */
				if (*(FruBuf+i+5+j) == ';') 
				{
					/* Append last element of array with '\0' to make it as string*/
					TempBuf[j] = '\0'; 
					break;
				}		
				else 
				{
					TempBuf[j] = *(FruBuf+i+5+j);
				}
			}
			
			if (0 == strncmp(TempBuf, "SB_XVI", 7))
			{
				strcpy (env_platformid, "Carnegie");
				break ;
			}

			if (0 == strncmp(TempBuf,"SB_XVIII", 9))
			{
				strcpy (env_platformid , "Absolut");
				break ;
			}
			if (0 == strncmp(TempBuf, "SB_XIX", 7))
			{
				strcpy (env_platformid, "Vespa");
				break ;
			}
			
		} 
		
	}
	if (i >= 256-6)
	{
		printf("Did not find platfromid in FruBuf\n");

#define myprintable(x) (((x)>=0x20)&&((x)<=0x7E))
		int j;
	    for (i = 0; i < 16; ++i) {
			for (j = 0; j < 16; j++) {
				printf ("%02X ", FruBuf[i * 16 + j]);
			}
			printf (" ");

			for (j = 0; j < 16; j++) {
				unsigned char   ch = (char)FruBuf[i * 16 + j];

				if (myprintable(ch)) {
					printf ("%c", ch);
				} else {
					printf ("-");
				}
			}
			printf ("\n");
		}

		return -1;
	}
	return 0;

}

#endif 

/*
 * Read the platformid  from EEPROM.
 * 
 */
unsigned int 
readplatformid ( char *buf  )
{
	char platformid [256] = "Reference";
	char *env_platformid = NULL;
	
#ifdef CFG_READ_MAC_FROM_FRU_SUPPORT

	/* Get the platformid from FRU EEPROM */
	int retval ;
	extern int ReadFru (unsigned char *);
	unsigned char FruBuf[256] = { 0 };
	char platformStr [256] = { 0 };

	/* Read FRU data */
	retval = ReadFru (FruBuf);

	if ( 0 == retval )
	{
		/* Obtain platformid from FRU data */
		retval = parsePlatformidAndSet (FruBuf,platformStr);
		if ( 0 == retval )
		{
			strcpy (buf,platformStr);	
			return strlen(platformStr);
	      	}	
	}
	else 
	{
		printf("WARNING: platformid could not be parsed from FRU, perhaps it is not programmed \n");
	}

#endif 	

	/* Get  the platformid from uboot environment variable , if  EEPROM dont have the platformid */
	env_platformid = getenv("platformid");
	if ( NULL != env_platformid )
	{
	       if ((0 == strncmp(env_platformid, "Reference", 9) ) ||( 0 == strncmp(env_platformid, "Carnegie", 8) )|| (0 == strncmp(env_platformid, "Absolut", 7)) || ( 0== strncmp(env_platformid, "Vespa", 5) ) ) 
		{
			strcpy(buf, env_platformid);
			return strlen(env_platformid);
		}	
	}	

	/* Set the default platform ID as Reference , if both EEPROM and uboot environment variable dont have the platformid  */
	strcpy (buf, platformid);
	return strlen(platformid);
}
