#include <config.h>
#include <common.h>
#include <net.h>
#include <asm/io.h>

#ifdef	CONFIG_VSC452

#include <vsc452/vsc452regs.h>

/* FIXME: we should convert these to in32 and out32 */
#define IO_WORD(offset)	     (*(volatile unsigned long *)(0xA0000000 | offset))

#define write32(addr, val)	writel(val, 0xa0000000 + (addr))
#define read32(addr)		readl(0xa0000000 + (addr))
#define GPIO_PORT0		0x10200a00
#define GPIO_PORT1		0x10200a40
#define GPIO_PORT2		0x10200a80
#define GPIO_PORT3		0x10200ac0
#define GPIO_PORT4		0x10200b00
#define GPIO_PORT5		0x10200b40
#define GPIO_PORT6		0x10200ba0
#define GPIO_OFFSET		4
#define GPIO(port, number)	GPIO_PORT0 + (port *0x40) + (number * GPIO_OFFSET)
#define CONSOLE_SELECTOR	GPIO(2,7)
unsigned int initialize_gpio(void);

void  initialize_serialReg(void)
{

       /*Initialising the ttyS0 registers */
       writel(0x00000080, SYS_GPURS0); /* GP Uart 0 receive status */
       writel(0x00000000, SYS_GPURD0); /* GP Uart 0 receive data */
       writel(0x00000003, SYS_GPUTS0); /* GP Uart 0 transmit status */
       writel(0x00000000, SYS_GPUTD0); /* GP Uart 0 transmit data */
       writel(0x00000048, SYS_GPUBR0); /* GP Uart 0 baud rate select */
       writel(0x00000000, SYS_GPUIE0); /* GP Uart 0 Interrupt enable */ // First disable interrupt bit and then clear the status
       writel(0x00000007, SYS_GPUIS0); /* GP Uart 0 Interrupt status */

       /*Initialising the ttyS1 registers */
       writel(0x00000000, SYS_GPURS1); /* GP Uart 1 receive status */
       writel(0x00000000, SYS_GPURD1); /* GP Uart 1 receive data */
       writel(0x00000003, SYS_GPUTS1); /* GP Uart 1 transmit status */
       writel(0x00000000, SYS_GPUTD1); /* GP Uart 1 transmit data */
       writel(0x00000048, SYS_GPUBR1); /* GP Uart 1 baud rate select */
       writel(0x00000000, SYS_GPUIE1); /* GP Uart 1 Interrupt enable */
       writel(0x00000007, SYS_GPUIS1); /* GP Uart 1 Interrupt status */

       return;
}
int serial_init(void)
{
	int bootdelay = 2;
	int abort = 0;
	int  ctrl_s = 0, ctrl_p = 0;

	serial_setbrg();
	initialize_gpio();	
	write32(CONSOLE_SELECTOR,0);
	initialize_serialReg();

	while ((bootdelay > 0) && (!abort)) {
		int i;

		--bootdelay;
		/* delay 100 * 10ms */
		for (i=0; !abort && i<100; ++i) {
            #define CTRL(x) (x-'@')
			if (tstc()) {	/* we got a key press	*/
			  switch (getc())
              {
                case CTRL('S'):
                    ctrl_s = 1;
                    break;
                case CTRL('P'):
                    ctrl_p=1;
                    break;
              }
            }
            if ((ctrl_s) && (ctrl_p))
	        {
			    abort  = 1;	
				bootdelay = 0;	/* no more delay	*/
				break;
			}
			udelay (10000);
		}
	}
	if (abort) {
		/* set GPIO to enable sp console */
		write32(CONSOLE_SELECTOR,1);
		printf("Enabling SP console...\n");
	}
	return 0;
}

void serial_setbrg(void)
{
	DECLARE_GLOBAL_DATA_PTR;

	switch(gd->baudrate)
	{
		case 9600:
			IO_WORD(SYS_GPUBR0) = (IO_WORD(SYS_GPUBR0) & SYS_GPUBRx_RATEMASK) | SYS_GPUBRx_9600;
			break;
		case 19200:
			IO_WORD(SYS_GPUBR0) = (IO_WORD(SYS_GPUBR0) & SYS_GPUBRx_RATEMASK) | SYS_GPUBRx_19200;
			break; 
		case 38400:
			IO_WORD(SYS_GPUBR0) = (IO_WORD(SYS_GPUBR0) & SYS_GPUBRx_RATEMASK) | SYS_GPUBRx_38400;
			break;
		case 57600:
			IO_WORD(SYS_GPUBR0) = (IO_WORD(SYS_GPUBR0) & SYS_GPUBRx_RATEMASK) | SYS_GPUBRx_57600;
			break;
		case 115200:
			IO_WORD(SYS_GPUBR0) = (IO_WORD(SYS_GPUBR0) & SYS_GPUBRx_RATEMASK) | SYS_GPUBRx_115200;
			break;
		default:
			printf("Baudrate change to %i unsupported by UART, change ignored\n",gd->baudrate);
	}
}

void serial_putc(const char c)
{
	if (c == '\n') serial_putc('\r');
	while (1)
	{
		if (IO_WORD(SYS_GPUTS0) & SYS_SITS_TXR)
		{
			IO_WORD(SYS_GPUTD0) = (unsigned int)c;
			return;
		}
	}
}

void serial_puts(const char * s)
{
	while (*s) {
		serial_putc(*s++);
	}
}

int serial_getc(void)
{

	while(! (IO_WORD(SYS_GPURS0) & SYS_GPURSx_RXR) )
		;
	return IO_WORD(SYS_GPURD0);
}

int serial_tstc(void)
{
	return IO_WORD(SYS_GPURS0) & SYS_GPURSx_RXR;
}

#endif	
